# Script to build a dataset to explore discounting trends

# imports
from pypeds import ipeds

# setup the years
years = list(range(2002, 2018))

## container for the dataset
df_orig = dict()

## build the dataset from the view
# for year in years:
#   x = views.tuition_discounting(fall_years=[year])
# --- RAISES ERROR ---
#   NameError: name 'x' is not defined

## build a dataset from the survey classes
x = ipeds.SFA(years=years)
x = x.extract()



# test the SFA class
from pypeds import ipeds


############### Test range of years

# the years to teset
years = list(range(2002, 2019))

# instantiate and go
tmp = ipeds.SFA(years=years)
tmp.extract()
sfa = tmp.load()
sfa.shape
sfa.fall_year.value_counts(dropna=False, sort=False)



"""A python package to help expedite insight generate from publicly available higher education datasets. """
__version__ = '0.102'

from pypeds.ipeds import *
from pypeds.datasets import *
from pypeds.views import *







# 
# #================================================== master dataset of institution attributes
# ## a dataframe with each rool as a school, and columns describing the institution
# def school_attributes(fall_years=[2017],
#                       hd_deg4yr = True,
#                       hd_service = False,
#                       hd_lower48 = False,
#                       hd_regions = True,
#                       hd_cols = ['unitid', 'fall_year', 'instnm', 'fips', 
#                                  'carnegie', 'sector', 'latitude', 'longitud'] ):
#   """
#   Build a dataset of school info and program completions.  
# 
#   Parameters:
#       years (list): a list of integers for the survey years to include for the migration data
#       hd_deg4yr (bool): boolean (default = True) as to filter to only include degree-granting 4-year institutions
#       hd_service (bool): boolean (default = True) which if True, will remove US service schools
#       hd_lower48 (bool): boolean (default = False) which if True, will only keep lower 48 states
#       hd_regions (bool): boolean (default = True) which if True, will add state and region data at the institutional level
#       hd_cols (list): a list of valid column names for the HD survey.  Only these columns will be returned.
#   """  
# 
#   # get the inst data
#   i = ipeds.HD(years=fall_years)
#   i.extract()
#   i.transform(deg4yr=hd_deg4yr)
#   i.transform(service=hd_service)
#   i.transform(lower_us=hd_lower48)
#   i.transform(regions=hd_regions)
#   i.transform(cols=hd_cols)
#   inst = i.load()
#   
#   # get inst characteristcs dataset
#   ic = ipeds.IC(years=fall_years)
#   ic.extract()
#   ic.transform()
#   



import pandas as pd
import numpy as np
import os
import requests
import zipfile
import glob


# zip file factory - returns a pandas dataframe
def zip_parser(url=None, survey=None):
    # setup the tmp path and file name
    # thanks to https://stackoverflow.com/questions/55718917/download-zip-file-locally-to-tempfile-extract-files-to-tempfile-and-list-the-f/55719124#55719124
    path = "/tmp/"
    file = survey + ".zip"
    survey_lower = survey.lower()
    # get the data
    try:
      results = requests.get(url)
    except:
      pass
    with open(path + file, 'wb') as f:
        f.write(results.content)
    # extract the files to the path
    file = zipfile.ZipFile(path + file)
    file.extractall(path=path)
    # list the csv files for the surveys, most likely get one , but may get to with _rv for revised
    files = glob.glob(path + "*"+survey_lower+"*")
    # isolate the file name
    if len(files) > 1:
        raw_file = [s for s in files if 'rv' in s]
        raw_file = str(raw_file[0]) # just in case, take first
    else:
        raw_file = str(files[0])
    # return a string
    return(str(raw_file))

def read_survey(path):
    if isinstance(path, list):
        path = path[0]
    # assumes a path, presumably from zip_parser
    try:
        ## encoding option needed for h2017, at least, wasnt needed for IC2013
        survey_file = pd.read_csv(path, encoding='ISO-8859-1')
    except:
        survey_file = pd.DataFrame({'path':path})
    # column names to lower - helps later and assumes a survey varname is historically unique
    survey_file.columns = survey_file.columns.str.lower()
    # add the survey
    return(survey_file)


###### utilities to crawl and return a big dataset for the survey
#
# def hd(years = None):
#     # returns a dataframe of 1 or more survey collections
#     # will always use the revised file _rv, if the file has it
#     assert isinstance(years, list), "year is not a list of integers"
#     # init a dataframe to append things to
#     hd_df = pd.DataFrame({'pypeds_init': [True]})
#     for year in years:
#         year_info = get_hd(year)
#         year_fpath = zip_parser(url=year_info['url'], survey=year_info['survey'])
#         tmp_df = read_survey(year_fpath)
#         tmp_df.columns = tmp_df.columns.str.lower()
#         tmp_df['survey_year'] = int(year)
#         tmp_df['fall_year'] = int(year)
#         hd_df = hd_df.append(tmp_df, ignore_index=True, sort=False)
#         # print("finished hd for year {}".format(str(year)))
#     # finish up
#     # ignore pandas SettingWithCopyWarning, basically
#     pd.options.mode.chained_assignment = None
#     hd_df_final = hd_df.loc[hd_df.pypeds_init != True, ]
#     hd_df_final.drop(columns=['pypeds_init'], inplace=True)
#     return(hd_df_final)
#
# def ic(years = None):
#     # returns a dataframe of 1 or more survey collections
#     # will always use the revised file _rv, if the file has it
#     assert isinstance(years, list), "year is not a list of integers"
#     # init dataframes to append things to
#     ic_df = pd.DataFrame({'pypeds_init': [True]})
#     adm_df = pd.DataFrame({'pypeds_init': [True]})
#     # loop for ic and conditional check for adm
#     for year in years:
#         year_info = get_ic(year)
#         year_fpath = zip_parser(url=year_info['url'], survey=year_info['survey'])
#         tmp_df = read_survey(year_fpath)
#         tmp_df.columns = tmp_df.columns.str.lower()
#         tmp_df['survey_year'] = int(year)
#         tmp_df['fall_year'] = int(year)
#         ic_df = ic_df.append(tmp_df, ignore_index=True, sort=False)
#         # check the year to get the admission data for 2014 and later
#         if year >= 2014:
#           year_info = get_adm(year)
#           year_fpath = zip_parser(url=year_info['url'], survey=year_info['survey'])
#           tmp_df = read_survey(year_fpath)
#           tmp_df.columns = tmp_df.columns.str.lower()
#           tmp_df['survey_year'] = int(year)
#           tmp_df['fall_year'] = int(year)
#           adm_df = adm_df.append(tmp_df, ignore_index=True, sort=False)
#
#     # finish up
#     # ignore pandas SettingWithCopyWarning,
#     pd.options.mode.chained_assignment = None
#     ic_df_final = ic_df.loc[ic_df.pypeds_init != True, ]
#     ic_df_final.drop(columns=['pypeds_init'], inplace=True)
#     adm_df_final = adm_df.loc[adm_df.pypeds_init != True, ]
#     adm_df_final.drop(columns=['pypeds_init'], inplace=True)
#     df = pd.merge(ic_df_final, adm_df_final, how="left", on=['unitid', 'survey_year'], suffixes=('_ic', '_adm'))
#     return(df)
#
# def sfa(years = None):
#     # returns a dataframe of 1 or more survey collections
#     # will always use the revised file _rv, if the file has it
#     assert isinstance(years, list), "year is not a list of integers"
#     # init a dataframe to append things to
#     sfa_df = pd.DataFrame({'pypeds_init': [True]})
#     for year in years:
#         year_info = get_sfa(year)
#         year_fpath = zip_parser(url=year_info['url'], survey=year_info['survey'])
#         tmp_df = read_survey(year_fpath)
#         tmp_df.columns = tmp_df.columns.str.lower()
#         tmp_df['survey_year'] = int(year)
#         tmp_df['fall_year'] = int(year-1)
#         sfa_df = sfa_df.append(tmp_df, ignore_index=True, sort=False)
#         # print("finished hd for year {}".format(str(year)))
#     # finish up
#     # ignore pandas SettingWithCopyWarning, basically
#     pd.options.mode.chained_assignment = None
#     sfa_df_final = sfa_df.loc[sfa_df.pypeds_init != True, ]
#     sfa_df_final.drop(columns=['pypeds_init'], inplace=True)
#     return(sfa_df_final)
#
# def efc(years = None):
#     # returns a dataframe of 1 or more survey collections
#     # will always use the revised file _rv, if the file has it
#     assert isinstance(years, list), "year is not a list of integers"
#     # init a dataframe to append things to
#     efc_df = pd.DataFrame({'pypeds_init': [True]})
#     for year in years:
#         year_info = get_efc(year)
#         year_fpath = zip_parser(url=year_info['url'], survey=year_info['survey'])
#         tmp_df = read_survey(year_fpath)
#         tmp_df.columns = tmp_df.columns.str.lower()
#         tmp_df['survey_year'] = int(year)
#         tmp_df['fall_year'] = int(year)
#         efc_df = efc_df.append(tmp_df, ignore_index=True, sort=False)
#         # print("finished hd for year {}".format(str(year)))
#     # finish up
#     # ignore pandas SettingWithCopyWarning, basically
#     pd.options.mode.chained_assignment = None
#     efc_df_final = efc_df.loc[efc_df.pypeds_init != True, ]
#     efc_df_final.drop(columns=['pypeds_init'], inplace=True)
#     return(efc_df_final)
#
# def icay(years = None):
#     # returns a dataframe of 1 or more survey collections
#     # will always use the revised file _rv, if the file has it
#     assert isinstance(years, list), "year is not a list of integers"
#     # init a dataframe to append things to
#     icay_df = pd.DataFrame({'pypeds_init': [True]})
#     for year in years:
#         year_info = get_icay(year)
#         year_fpath = zip_parser(url=year_info['url'], survey=year_info['survey'])
#         tmp_df = read_survey(year_fpath)
#         tmp_df.columns = tmp_df.columns.str.lower()
#         tmp_df['survey_year'] = int(year)
#         tmp_df['fall_year'] = int(year)
#         icay_df = icay_df.append(tmp_df, ignore_index=True, sort=False)
#         # print("finished hd for year {}".format(str(year)))
#     # finish up
#     # ignore pandas SettingWithCopyWarning, basically
#     pd.options.mode.chained_assignment = None
#     icay_df_final = icay_df.loc[icay_df.pypeds_init != True, ]
#     icay_df_final.drop(columns=['pypeds_init'], inplace=True)
#     return(icay_df_final)
#



from setuptools import setup

setup(name='pypeds',
      version='0.1.5',
      python_requires='>=3.6',
      description='Python package to work with IPEDS and other higher education datasets.',
      url='https://brocktibert.com/',
      author='@brocktibert',
      author_email='btibert3@gmail.com',
      license='MIT',
      packages=['pypeds'],
      zip_safe=False,
      include_package_data=True,
      install_requires=['pandas',
                        'requests',
                        'altair',
                        'dfply',
                        'numpy'])



import pandas as pd
from pypeds import ipeds

## test multiple years
YEARS = [2016,2017]

## test ic
ic = ipeds.IC(years=YEARS)
ic.extract()
df = ic.load()
df.survey_year.value_counts()

## test hd
hd = ipeds.HD(years=YEARS)
hd.extract()
df = hd.load()
df.survey_year.value_counts()

## test sfa
sfa = ipeds.SFA(YEARS)
sfa.extract()
df = sfa.load()
df.survey_year.value_counts()

## test efc
efc = ipeds.EFC(YEARS)
efc.extract()
df = efc.load()
df.survey_year.value_counts()

## test icay
icay = ipeds.ICAY(YEARS)
icay.extract()
df = icay.load()
df.survey_year.value_counts()



import pickle
import pandas as pd

## site 1

## the exported files from the private R github thrown into Downloads
edges = pd.read_csv("/Users/btibert/Downloads/site1-links.csv")
majors = pd.read_csv("/Users/btibert/Downloads/site1-majors.csv")

## put into a dict
network1 = dict(edges = edges, majors = majors)

## write out the pickle for use/load in pypeds
with open("data/network1.pickle", "wb") as handle:
  pickle.dump(network1, handle, protocol=pickle.HIGHEST_PROTOCOL)
  

## site 2

## the exported files from the private R github thrown into Downloads
edges = pd.read_csv("/Users/btibert/Downloads/site2-edges.csv")
mission = pd.read_csv("/Users/btibert/Downloads/site2-mission.csv")

## put into a dict
network2 = dict(edges = edges, mission = mission)

## write out the pickle for use/load in pypeds
with open("data/network2.pickle", "wb") as handle:
  pickle.dump(network2, handle, protocol=pickle.HIGHEST_PROTOCOL)



# test the SFA class
from pypeds import views



############### Test range of years

# simple test
years = list(range(2016, 2019, 2))

# go for it
x = views.migration(years=years)



# larger test
years = list(range(2002, 2019, 2))

# go for it
x = views.migration(years=years)

# error pops up but still good data, somewhere in IC that isnt present when just
# using HD class for 2002 to 2018.  Bizarre
## TODO:  explore the field and specify on import
###
# sys:1: DtypeWarning: Columns (12) have mixed types. 
# Specify dtype option on import or set low_memory=False.



# lower level features to collect ipeds survey datasets
import pandas as pd
import os
import requests
import zipfile
import glob
import time
import datetime
from dfply import *
from pypeds import datasets

# ================================= core features

# zip file factory - returns a pandas dataframe
def zip_parser(url=None, survey=None):
    # setup the tmp path and file name
    # thanks to https://stackoverflow.com/questions/55718917/download-zip-file-locally-to-tempfile-extract-files-to-tempfile-and-list-the-f/55719124#55719124
    # path = "/tmp/pypeds/" + str(int(time.time())) + "/"  # hacky way to make unique path to extract time
    _today = datetime.datetime.today().strftime('%Y%m%d')
    survey_lower = survey.lower()
    path = "/tmp/" + str(_today) + str(survey) + "/"  # hacky way to make unique path to extract date and survey
    file = survey + ".zip"
    
    # naive way to do cacheing - if the path for today exists, dont do anything, if it doesnt, get the data
    if not os.path.exists(path + file):
        # get the data
        os.mkdir(path)
        try:
            results = requests.get(url)
        except:
            pass
        with open(path + file, 'wb') as f:
            f.write(results.content)

    # extract the files to the path
    file = zipfile.ZipFile(path + file)
    file.extractall(path=path)
    # list the csv files for the surveys, most likely get one , but may get to with _rv for revised
    files = glob.glob(path + "*" + survey_lower + "*")
    files = [x.lower() for x in files]
    # isolate the file name
    if len(files) > 1:
        raw_file = [s for s in files if 'rv' in s]
        raw_file = str(raw_file[0])  # just in case, take first
    else:
        raw_file = str(files[0])
    # return a string
    return (str(raw_file))


def read_survey(path):
    if isinstance(path, list):
        path = path[0]
    # assumes a path, presumably from zip_parser
    try:
        ## encoding option needed for h2017, at least, wasnt needed for IC2013
        survey_file = pd.read_csv(path, encoding='ISO-8859-1')
    except:
        survey_file = pd.DataFrame({'path': path})
    # remove the file
    os.remove(path)
    # column names to lower - helps later and assumes a survey varname is historically unique
    survey_file.columns = survey_file.columns.str.lower()
    # add the survey
    return (survey_file)


# ================================= utilities to build url data

# build a valid ipeds survey url - return a dict with a survey key and url for download
def get_hd(year):
    # assert that year is a int and length 1
    assert isinstance(year, int), "year is not an integer"
    assert year >= 2002 and year <= 2018, "year must be >=2002 and < 2018"
    # build the SURVEY id
    SURVEY = 'HD' + str(year)
    # build the url
    URL = "https://nces.ed.gov/ipeds/datacenter/data/{}.zip".format(SURVEY)
    # return the bits as a dictionary for use later
    return ({'url': URL, 'survey': SURVEY})


def get_ic(year):
    # assert that year is a int and length 1
    assert isinstance(year, int), "year is not an integer"
    assert year >= 2002 and year <= 2018, "year must be >=2002 and < 2018"
    # build the SURVEY id
    SURVEY = 'IC' + str(year)
    # build the url
    URL = "https://nces.ed.gov/ipeds/datacenter/data/{}.zip".format(SURVEY)
    # return the bits as a dictionary for use later
    return ({'url': URL, 'survey': SURVEY})


def get_adm(year):
    # assert that year is a int and length 1
    assert isinstance(year, int), "year is not an integer"
    assert year >= 2014 and year <= 2018, "year must be >=2014 and < 2018"
    # build the SURVEY id
    SURVEY = 'ADM' + str(year)
    # build the url
    URL = "https://nces.ed.gov/ipeds/datacenter/data/{}.zip".format(SURVEY)
    # return the bits as a dictionary for use later
    return ({'url': URL, 'survey': SURVEY})


def get_sfa(year):
    # assert that year is a int and length 1
    assert isinstance(year, int), "year is not an integer"
    assert year >= 2002 and year <= 2018, "year must be >=2002 and < 2018"
    # build the SURVEY id
    sfa_year = str(year - 1)[2:] + str(year)[2:]
    SURVEY = 'SFA' + str(sfa_year)
    # build the url
    URL = "https://nces.ed.gov/ipeds/datacenter/data/{}.zip".format(SURVEY)
    # return the bits as a dictionary for use later
    return ({'url': URL, 'survey': SURVEY})


def get_efc(year):
    # assert that year is a int and length 1
    assert isinstance(year, int), "year is not an integer"
    assert year >= 2002 and year <= 2018, "year must be >=2002 and < 2018"
    # build the SURVEY id
    SURVEY = 'EF' + str(year) + "C"
    # build the url
    URL = "https://nces.ed.gov/ipeds/datacenter/data/{}.zip".format(SURVEY)
    # return the bits as a dictionary for use later
    return ({'url': URL, 'survey': SURVEY})


def get_icay(year):
    # assert that year is a int and length 1
    assert isinstance(year, int), "year is not an integer"
    assert year >= 2002 and year <= 2018, "year must be >=2002 and < 2018"
    # build the SURVEY id
    SURVEY = 'IC' + str(year) + "_AY"
    # build the url
    URL = "https://nces.ed.gov/ipeds/datacenter/data/{}.zip".format(SURVEY)
    # return the bits as a dictionary for use later
    return ({'url': URL, 'survey': SURVEY})

def get_om(year):
    # assert that year is a int and length 1
    assert isinstance(year, int), "year is not an integer"
    assert year >= 2015 and year <= 2018, "year must be >=2015 and < 2018"
    # build the SURVEY id
    SURVEY = 'OM' + str(year) 
    # build the url
    URL = "https://nces.ed.gov/ipeds/datacenter/data/{}.zip".format(SURVEY)
    # return the bits as a dictionary for use later
    return ({'url': URL, 'survey': SURVEY})

def get_efd(year):
    # assert that year is a int and length 1
    assert isinstance(year, int), "year is not an integer"
    assert year >= 2002 and year <= 2018, "year must be >=2002 and < 2018"
    # build the SURVEY id
    SURVEY = 'EF' + str(year)  + "D"
    # build the url
    URL = "https://nces.ed.gov/ipeds/datacenter/data/{}.zip".format(SURVEY)
    # return the bits as a dictionary for use later
    return ({'url': URL, 'survey': SURVEY})

def get_ff1(year):
    # assert that year is a int and length 1
    assert isinstance(year, int), "year is not an integer"
    assert year >= 2002 and year <= 2018, "year must be >=2002 and < 2018"
    # build the SURVEY id
    ff1_year = str(year - 1)[2:] + str(year)[2:]
    SURVEY = 'F' + str(ff1_year)  + "_F1A"
    # build the url
    URL = "https://nces.ed.gov/ipeds/datacenter/data/{}.zip".format(SURVEY)
    # return the bits as a dictionary for use later
    return ({'url': URL, 'survey': SURVEY})

def get_ff2(year):
    # assert that year is a int and length 1
    assert isinstance(year, int), "year is not an integer"
    assert year >= 2002 and year <= 2018, "year must be >=2002 and < 2018"
    # build the SURVEY id
    ff2_year = str(year - 1)[2:] + str(year)[2:]
    SURVEY = 'F' + str(ff2_year)  + "_F2"
    # build the url
    URL = "https://nces.ed.gov/ipeds/datacenter/data/{}.zip".format(SURVEY)
    # return the bits as a dictionary for use later
    return ({'url': URL, 'survey': SURVEY})

def get_ca(year):
    # assert that year is a int and length 1
    assert isinstance(year, int), "year is not an integer"
    assert year >= 2002 and year <= 2018, "year must be >=2002 and < 2018"
    # build the SURVEY id
    SURVEY = 'C' + str(year)  + "_A"
    # build the url
    URL = "https://nces.ed.gov/ipeds/datacenter/data/{}.zip".format(SURVEY)
    # return the bits as a dictionary for use later
    return ({'url': URL, 'survey': SURVEY})


# ================================= build the classes

class HD(object):
    """
    Directory Information from the Institutional Characteristics survey.
    Currently supports the years 2002 - 2018.

    Methods are extract, transform, and load.
    """

    def __init__(self, years=[2017]):
        """
        The constructor for the HD survey

        Parameters:
          years (list): List of ints for the survey year
        """

        self.years = years
        self.df = pd.DataFrame()

    def extract(self):
        """
        Method to pull one or more IC surveys based on the configured object

        The extract method currently supports back to 2002 and up to 2017.
        """

        init_df = pd.DataFrame({'pypeds_init': [True]})
        for year in self.years:
            # assert that year is a int and length 1
            assert isinstance(year, int), "year is not an integer"
            assert year >= 2002 and year <= 2018, "year must be >=2002 and < 2018"
            # build the SURVEY id
            SURVEY = 'HD' + str(year)
            # build the url
            URL = "https://nces.ed.gov/ipeds/datacenter/data/{}.zip".format(SURVEY)
            # return the bits as a dictionary for use later
            year_info = {'url': URL, 'survey': SURVEY}
            # year_info = get_efc(year)
            year_fpath = zip_parser(url=year_info['url'], survey=year_info['survey'])
            tmp_df = read_survey(year_fpath)
            tmp_df.columns = tmp_df.columns.str.lower()
            tmp_df.columns = tmp_df.columns.str.strip()
            tmp_df['survey_year'] = int(year)
            tmp_df['fall_year'] = int(year)
            init_df = init_df.append(tmp_df, ignore_index=True, sort=False)
            # print("finished hd for year {}".format(str(year)))
        # finish up
        # ignore pandas SettingWithCopyWarning, basically
        pd.options.mode.chained_assignment = None
        init_df = init_df.loc[init_df.pypeds_init != True,]
        init_df.drop(columns=['pypeds_init'], inplace=True)
        # return(init_df)
        self.df = self.df.append(init_df, ignore_index=True)

    # method to return the data
    def load(self):
        """
        The load method returns a pandas dataframe that has been extracted, and optionally, transformed.
        """

        return (self.df)

    def transform(self, 
                  deg4yr=None, 
                  service=None, 
                  lower_us=None, 
                  regions = None, 
                  cols=None):
        """
        The transformation methods for the dataset collected.  
        Arguments activate the transformation, otherwise they are not performed.

        Parameters:
            deg4yr (bool): if True, keep only public/private non profit 4-year that are degree granting
            service (bool): if True, remove service schools
            lower_us (bool): if True, keep only the contintental 48 states incl. D.C.
            regions (bool): if True, add state/region info to the institution
            cols (list): A list of valid column names to keep, all others will be excluded
        """

        tmpdf = self.df

        # degree granting non profit private 4yr and public 4 yr
        if deg4yr:
            tmp = tmpdf
            tmp_f = tmp.loc[(tmp.sector.isin([1, 2])) & (tmp.pset4flg == 1) & (tmp.deggrant == 1), ]
            tmpdf = tmp_f

        # remove service schools
        if service:
            tmp = tmpdf
            tmp_f = tmp.loc[tmp.obereg != 0, ]
            tmpdf = tmp_f

        # lower 48 states with DC
        if lower_us:
            tmp = tmpdf
            tmp_f = tmp.loc[tmp.fips <= 51, ]
            tmp_f = tmp.loc[tmp.fips != 2, ]
            tmp_f = tmp.loc[tmp.fips != 12, ]
            tmpdf = tmp_f
        
        # add the regions info
        if regions:
            r = datasets.region_xwalk()
            r = r >> select(['fips','name','ipeds_region'])
            r = r.rename(columns={"name": "state_name"})
            tmp = tmpdf
            tmp_f = pd.merge(left=tmp, right=r, on="fips", how="left")
            tmpdf = tmp_f

        # select columns
        if cols is not None:
            assert isinstance(cols, list), 'the argument cols must be a list'
            if len(cols) > 0:
                tmp = tmpdf
                tmp_f = tmp >> select(cols)
                tmpdf = tmp_f

        # return the data
        self.df = tmpdf


class IC(object):
    """
    Educational offerings, organization, services and athletic associations from the Institutional Characteristics survey.
    Currently support the years 2002 to 2018.
    """

    # init
    def __init__(self, years=[2018]):
        """
        The constructor for the IC survey

        Parameters:
          years (list): List of ints for the survey year
        """

        self.years = years
        self.df = pd.DataFrame()

    # method to get the data and return a dataframe
    def extract(self):
        """
        Method to pull one or more IC surveys based on the configured object

        The extract method currently supports back to 2002 and accounts for the application data being broken
        out of the IC survey starting in 2014, in which the survey prefix is ADM.
        """

        ic_df = pd.DataFrame({'pypeds_init': [True]})
        adm_df = pd.DataFrame({'pypeds_init': [True]})
        # loop for ic and conditional check for adm
        for year in self.years:
            # the original dataset
            year_info = get_ic(year)
            year_fpath = zip_parser(url=year_info['url'], survey=year_info['survey'])
            tmp_df = read_survey(year_fpath)
            tmp_df.columns = tmp_df.columns.str.lower()
            tmp_df.columns = tmp_df.columns.str.strip()
            tmp_df['survey_year'] = int(year)
            tmp_df['fall_year'] = int(year)
            ic_df = ic_df.append(tmp_df, ignore_index=True, sort=False)
            # check the year to get the admission data for 2014 and later
            # this is in addition to above
            if year >= 2014:
                year_info = get_adm(year)
                year_fpath = zip_parser(url=year_info['url'], survey=year_info['survey'])
                tmp_df = read_survey(year_fpath)
                tmp_df.columns = tmp_df.columns.str.lower()
                tmp_df['survey_year'] = int(year)
                tmp_df['fall_year'] = int(year)
                adm_df = adm_df.append(tmp_df, ignore_index=True, sort=False)

        # finish up
        # ignore pandas SettingWithCopyWarning,
        pd.options.mode.chained_assignment = None
        ic_df_final = ic_df.loc[ic_df.pypeds_init != True,]
        ic_df_final.drop(columns=['pypeds_init'], inplace=True)
        adm_df_final = adm_df.loc[adm_df.pypeds_init != True,]
        adm_df_final.drop(columns=['pypeds_init'], inplace=True)
        # df = pd.merge(ic_df_final, adm_df_final, 
        #               how="left", 
        #               on=['unitid', 'survey_year'], 
        #               suffixes=('_ic', '_adm'))
        df = pd.merge(ic_df_final, adm_df_final,
                      how="left",
                      on=['unitid', 'survey_year', 'fall_year'])
        self.df = self.df.append(df, ignore_index=True)

    def load(self):
        """
        The load method returns a pandas dataframe that has been extracted, and optionally, transformed.
        """

        return (self.df)

    def transform(self, admit_rate=True, yield_rate=True, app_data=None, cols=None):
        """
        The transformation method of the data.  
        Arguments activate the transformation, otherwise they are not performed.

        Parameters:
            admit_rate (bool): if True, add the admit rate calculation as a column
            admit_rate (bool): if True, add yield rate calculation as a column
            app_data (bool): if True, filter out records missing using the column applcn`
            cols (list): A list of valid column names to keep, all others will be excluded
        """

        tmpdf = self.df

        # calc admit rate
        if admit_rate:
            tmp = tmpdf
            tmp['admit_rate'] = tmp['admssn'] / tmp['applcn']
            tmpdf = tmp
        
        # calc yield rate
        if yield_rate:
            tmp = tmpdf
            tmp['yield_rate'] = tmp['enrlt'] / tmp['admssn']
            tmpdf = tmp
        
        # keep those with adm survey data not missing
        if app_data:
            tmp = tmpdf
            tmp_f = tmp.dropna(subset=['applcn'], inplace=False)
            tmpdf = tmp_f

        # select columns
        if cols is not None:
            assert isinstance(cols, list), 'cols must be a list'
            if len(cols) > 0:
                tmp = tmpdf
                tmp_f = tmp >> select(cols)
                tmpdf = tmp_f
        
        # return the data
        self.df = tmpdf



class SFA(object):
    """
    Student financial aid and net price from the Student Financial Aid and Net Price survey.
    """

    def __init__(self, years=[2017]):
        """
        The constructor for the SFA survey

        Parameters:
          years (list): List of ints for the survey year
        """

        self.years = years
        self.df = pd.DataFrame()

    def extract(self, status = None):
        """
        Method to pull one or more SFA surveys based on the configured object

        The extract method currently supports back to 2002
        """

        init_df = pd.DataFrame({'pypeds_init': [True]})
        for year in self.years:
            # since we use numpy, convert to int
            year = int(year)
            if status:
                print("Starting " + str(year))
            year_info = get_sfa(year)
            year_fpath = zip_parser(url=year_info['url'], survey=year_info['survey'])
            tmp_df = read_survey(year_fpath)
            tmp_df.columns = tmp_df.columns.str.lower()
            tmp_df.columns = tmp_df.columns.str.strip()
            tmp_df['survey_year'] = int(year)
            tmp_df['fall_year'] = int(year) - 1
            init_df = init_df.append(tmp_df, ignore_index=True, sort=False)
            # print("finished hd for year {}".format(str(year)))
        # finish up
        # ignore pandas SettingWithCopyWarning, basically
        pd.options.mode.chained_assignment = None
        init_df = init_df.loc[init_df.pypeds_init != True,]
        init_df.drop(columns=['pypeds_init'], inplace=True)
        # return(init_df)
        self.df = self.df.append(init_df, ignore_index=True)

        # method to return the data
    def load(self):
        """
        The load method returns a pandas dataframe that has been extracted, and optionally, transformed.
        """

        return (self.df)

    def transform(self, cols=None):
        """
        The transformation method of the data.  
        Arguments activate the transformation, otherwise they are not performed.

        Parameters:
            cols (list): A list of valid column names to keep, all others will be excluded
        """
        
        tmpdf = self.df

        # select columns
        # TODO: conditionally check for net price columns
        #       either as a separate filter or specific variable names
        if cols is not None:
            assert isinstance(cols, list), 'cols must be a list'
            if len(cols) > 0:
                tmp = tmpdf
                tmp_f = tmp >> select(cols)
                tmpdf = tmp_f

        # return the data
        self.df = tmpdf





class EFC(object):
    """
    Residence and migration of first-time freshman from the Fall Enrollment survey.
    """

    def __init__(self, years=[2017]):
        """
        The constructor for the EF_C survey

        Parameters:
          years (list): List of ints for the survey year
        """

        self.years = years
        self.df = pd.DataFrame()

    def extract(self):
        """
        Method to pull one or more EF_C surveys based on the configured object
        """

        init_df = pd.DataFrame({'pypeds_init': [True]})
        for year in self.years:
            year_info = get_efc(year)
            year_fpath = zip_parser(url=year_info['url'], survey=year_info['survey'])
            tmp_df = read_survey(year_fpath)
            tmp_df.columns = tmp_df.columns.str.lower()
            tmp_df.columns = tmp_df.columns.str.strip()
            tmp_df['survey_year'] = int(year)
            tmp_df['fall_year'] = int(year)
            init_df = init_df.append(tmp_df, ignore_index=True, sort=False)
        # finish up
        # ignore pandas SettingWithCopyWarning, basically
        pd.options.mode.chained_assignment = None
        init_df = init_df.loc[init_df.pypeds_init != True,]
        init_df.drop(columns=['pypeds_init'], inplace=True)
        # return(init_df)
        self.df = self.df.append(init_df, ignore_index=True)

    def load(self):
        """
        The load method returns a pandas dataframe that has been extracted, and optionally, transformed.
        """

        return (self.df)

    def transform(self, state=None, line=None, cols=None):
        """
        The transformation method of the data.  
        Arguments activate the transformation, otherwise they are not performed.

        Parameters:
            state (list): a list of valid numeric codes, one for each state in the efcstate field
            line (list): a list of valid numeric codes to filter the line field
            cols (list): a list of the columns to be kept, column names in quotes
        """

        tmpdf = self.df
        
        # filter rows by efcstate
        if state is not None:
            assert isinstance(state, list), 'state must a list'
            if len(state) > 0:
                tmp = tmpdf
                tmp_f = tmp.loc[tmp.efcstate.isin(state)]
                tmpdf = tmp_f


        # filter rows by line
        if line is not None:
            assert isinstance(line, list), 'line must a list'
            if len(line) > 0:
                tmp = tmpdf
                tmp_f = tmp.loc[tmp.line.isin(line)]
                tmpdf = tmp_f
        

        # select columns
        if cols is not None:
            assert isinstance(cols, list), 'cols must be a list'
            if len(cols) > 0:
                tmp = tmpdf
                tmp_f = tmp >> select(cols)
                tmpdf = tmp_f
        
        # return the data
        self.df = tmpdf


class ICAY(object):
    """
    Student charges for academic year programs from the Institutional Characteristics survey.
    """

    def __init__(self, years=[2017]):
        """
        The constructor for the IC_AY survey

        Parameters:
          years (list): List of ints for the survey year
        """

        self.years = years
        self.df = pd.DataFrame()

    def extract(self):
        """
        Method to pull one or more IC_AY surveys based on the configured object
        """

        init_df = pd.DataFrame({'pypeds_init': [True]})
        for year in self.years:
            year_info = get_icay(year)
            year_fpath = zip_parser(url=year_info['url'], survey=year_info['survey'])
            tmp_df = read_survey(year_fpath)
            tmp_df.columns = tmp_df.columns.str.lower()
            tmp_df.columns = tmp_df.columns.str.strip()
            tmp_df['survey_year'] = int(year)
            tmp_df['fall_year'] = int(year)
            init_df = init_df.append(tmp_df, ignore_index=True, sort=False)
        # finish up
        # ignore pandas SettingWithCopyWarning, basically
        pd.options.mode.chained_assignment = None
        init_df = init_df.loc[init_df.pypeds_init != True,]
        init_df.drop(columns=['pypeds_init'], inplace=True)
        # return(init_df)
        self.df = self.df.append(init_df, ignore_index=True)

    def load(self):
        """
        The load method returns a pandas dataframe that has been extracted, and optionally, transformed.
        """

        return (self.df)

    def transform(self, cols=None):
        """
        The transformation method of the data.  
        Arguments activate the transformation, otherwise they are not performed.

        Parameters:
            cols (list): a list of the columsn to be kept, column names in quotes
        """
        
        tmpdf = self.df
        
        # filter the columns
        if cols is not None:
            assert isinstance(cols, list), 'cols must be a list'
            if len(cols) > 0:
                tmp = tmpdf
                tmp_f = tmp >> select(cols)
                tmpdf = tmp_f
        
        # return the dataset
        self.df = tmpdf
        

class OM(object):
    """
    Award and enrollment data at four, six and eight years of entering degree/certificate-seeking undergraduate cohorts at degree-granting institutions, by Pell status
    """

    def __init__(self, years=[2017]):
        """
        The constructor for the IC_AY survey

        Parameters:
          years (list): List of ints for the survey year
        """

        self.years = years
        self.df = pd.DataFrame()

    def extract(self):
        """
        Method to pull one or more IC_AY surveys based on the configured object
        """

        init_df = pd.DataFrame({'pypeds_init': [True]})
        for year in self.years:
            year_info = get_om(year)
            year_fpath = zip_parser(url=year_info['url'], survey=year_info['survey'])
            tmp_df = read_survey(year_fpath)
            tmp_df.columns = tmp_df.columns.str.lower()
            tmp_df.columns = tmp_df.columns.str.strip()
            tmp_df['survey_year'] = int(year)
            tmp_df['fall_year'] = int(year) - 8
            init_df = init_df.append(tmp_df, ignore_index=True, sort=False)
        # finish up
        # ignore pandas SettingWithCopyWarning, basically
        pd.options.mode.chained_assignment = None
        init_df = init_df.loc[init_df.pypeds_init != True,]
        init_df.drop(columns=['pypeds_init'], inplace=True)
        # return(init_df)
        self.df = self.df.append(init_df, ignore_index=True)

    def load(self):
        """
        The load method returns a pandas dataframe that has been extracted, and optionally, transformed.
        """

        return (self.df)

class EFD(object):
    """
    Total entering class, retention rates, and student-to-faculty ratio
    """

    def __init__(self, years=[2017]):
        """
        The constructor for the IC_AY survey

        Parameters:
          years (list): List of ints for the survey year
        """

        self.years = years
        self.df = pd.DataFrame()

    def extract(self):
        """
        Method to pull one or more IC_AY surveys based on the configured object
        """

        init_df = pd.DataFrame({'pypeds_init': [True]})
        for year in self.years:
            year_info = get_efd(year)
            year_fpath = zip_parser(url=year_info['url'], survey=year_info['survey'])
            tmp_df = read_survey(year_fpath)
            tmp_df.columns = tmp_df.columns.str.lower()
            tmp_df.columns = tmp_df.columns.str.strip()
            tmp_df['survey_year'] = int(year)
            tmp_df['fall_year'] = int(year)
            init_df = init_df.append(tmp_df, ignore_index=True, sort=False)
        # finish up
        # ignore pandas SettingWithCopyWarning, basically
        pd.options.mode.chained_assignment = None
        init_df = init_df.loc[init_df.pypeds_init != True,]
        init_df.drop(columns=['pypeds_init'], inplace=True)
        # return(init_df)
        self.df = self.df.append(init_df, ignore_index=True)

    def load(self):
        """
        The load method returns a pandas dataframe that has been extracted, and optionally, transformed.
        """

        return (self.df)


class FF1(object):
    """
    Private not-for-profit institutions or Public institutions using FASB:
    """

    def __init__(self, years=[2018]):
        """
        Public institutions - GASB

        Parameters:
          years (list): List of ints for the survey year
        """

        self.years = years
        self.df = pd.DataFrame()

    def extract(self):
        """
        Method to pull one or more IC_AY surveys based on the configured object
        """

        init_df = pd.DataFrame({'pypeds_init': [True]})
        for year in self.years:
            year = int(year)
            year_info = get_ff1(year)
            year_fpath = zip_parser(url=year_info['url'], survey=year_info['survey'])
            tmp_df = read_survey(year_fpath)
            tmp_df.columns = tmp_df.columns.str.lower()
            tmp_df.columns = tmp_df.columns.str.strip()
            tmp_df['survey_year'] = int(year)
            tmp_df['fall_year'] = int(year) -1
            init_df = init_df.append(tmp_df, ignore_index=True, sort=False)
        # finish up
        # ignore pandas SettingWithCopyWarning, basically
        pd.options.mode.chained_assignment = None
        init_df = init_df.loc[init_df.pypeds_init != True,]
        init_df.drop(columns=['pypeds_init'], inplace=True)
        # return(init_df)
        self.df = self.df.append(init_df, ignore_index=True)

    def load(self):
        """
        The load method returns a pandas dataframe that has been extracted, and optionally, transformed.
        """

        return (self.df)

    def transform(self, cols=None):
        """
        The transformation method of the data.  
        Arguments activate the transformation, otherwise they are not performed.

        Parameters:
            cols (list): a list of the columsn to be kept, column names in quotes
        """
        
        tmpdf = self.df
        
        # filter the columns
        if cols is not None:
            assert isinstance(cols, list), 'cols must be a list'
            if len(cols) > 0:
                tmp = tmpdf
                tmp_f = tmp >> select(cols)
                tmpdf = tmp_f
        
        # return the dataset
        self.df = tmpdf


class FF2(object):
    """
    Private not-for-profit institutions or Public institutions using FASB:
    """

    def __init__(self, years=[2018]):
        """
        Public institutions - GASB

        Parameters:
          years (list): List of ints for the survey year
        """

        self.years = years
        self.df = pd.DataFrame()

    def extract(self):
        """
        Method to pull one or more IC_AY surveys based on the configured object
        """

        init_df = pd.DataFrame({'pypeds_init': [True]})
        for year in self.years:
            year = int(year)
            year_info = get_ff2(year)
            year_fpath = zip_parser(url=year_info['url'], survey=year_info['survey'])
            tmp_df = read_survey(year_fpath)
            tmp_df.columns = tmp_df.columns.str.lower()
            tmp_df.columns = tmp_df.columns.str.strip()
            tmp_df['survey_year'] = int(year)
            tmp_df['fall_year'] = int(year) -1
            init_df = init_df.append(tmp_df, ignore_index=True, sort=False)
        # finish up
        # ignore pandas SettingWithCopyWarning, basically
        pd.options.mode.chained_assignment = None
        init_df = init_df.loc[init_df.pypeds_init != True,]
        init_df.drop(columns=['pypeds_init'], inplace=True)
        # return(init_df)
        self.df = self.df.append(init_df, ignore_index=True)

    def load(self):
        """
        The load method returns a pandas dataframe that has been extracted, and optionally, transformed.
        """

        return (self.df)

    def transform(self, cols=None):
        """
        The transformation method of the data.  
        Arguments activate the transformation, otherwise they are not performed.

        Parameters:
            cols (list): a list of the columsn to be kept, column names in quotes
        """
        
        tmpdf = self.df
        
        # filter the columns
        if cols is not None:
            assert isinstance(cols, list), 'cols must be a list'
            if len(cols) > 0:
                tmp = tmpdf
                tmp_f = tmp >> select(cols)
                tmpdf = tmp_f
        
        # return the dataset
        self.df = tmpdf


class C_A(object):
    """
    Awards/degrees conferred by program (6-digit CIP code), award level, race/ethnicity, and gender
    """

    def __init__(self, years=[2018]):
        """
        Public institutions - GASB

        Parameters:
          years (list): List of ints for the survey year
        """

        self.years = years
        self.df = pd.DataFrame()

    def extract(self):
        """
        Method to pull one or more IC_AY surveys based on the configured object
        """

        init_df = pd.DataFrame({'pypeds_init': [True]})
        for year in self.years:
            year = int(year)
            year_info = get_ca(year)
            year_fpath = zip_parser(url=year_info['url'], survey=year_info['survey'])
            tmp_df = read_survey(year_fpath)
            tmp_df.columns = tmp_df.columns.str.lower()
            tmp_df.columns = tmp_df.columns.str.strip()
            tmp_df['survey_year'] = int(year)
            tmp_df['fall_year'] = int(year) -1
            init_df = init_df.append(tmp_df, ignore_index=True, sort=False)
        # finish up
        # ignore pandas SettingWithCopyWarning, basically
        pd.options.mode.chained_assignment = None
        init_df = init_df.loc[init_df.pypeds_init != True,]
        init_df.drop(columns=['pypeds_init'], inplace=True)
        # return(init_df)
        self.df = self.df.append(init_df, ignore_index=True)

    def load(self):
        """
        The load method returns a pandas dataframe that has been extracted, and optionally, transformed.
        """

        return (self.df)

    def transform(self, 
                  cip_label=True, 
                  award_level=True, 
                  first_major=True,
                  grand_total=False,
                  level_keep=None,
                  cols=None):
        """
        The transformation method of the data.  
        Arguments activate the transformation, otherwise they are not performed.

        Parameters:
            cip_label (bool): Add the 2010 cip code labels.  Default is True.
            award_level (bool): Add the labels for the award levels.  Default is True.
            first_major (bool): If True (default), filter rows where majornum ==  1 for first major
            grand_total (bool): Should the Grand Total cip code be included? Default is False.
            level_keep (list): a list of the award level codes to be kept. Note, this takes the numeric code, not the label.  For help, refer to datasets.award_levels().
            cols (list): a list of the columns to be kept, column names in quotes
        """
        
        tmpdf = self.df
        
        # add the CIP code labels
        if cip_label:
            # get the cip code crosswalk
            cips = datasets.cipcodes()
            # add the cip codes
            tmp = tmpdf
            tmp = pd.merge(left=tmp, right=cips, on="cipcode", how="left")
            # set the update
            tmpdf = tmp
        
        # add the award level labels
        if award_level:
            # get the award level labels
            al = datasets.award_levels()
            # add the labels onto the dataframe
            tmp = tmpdf
            tmp = pd.merge(left=tmp, right=al, on="awlevel", how="left")
            # set the update
            tmpdf = tmp

        # keep only the first major
        if first_major:
            tmp = tmpdf
            tmp = tmp.loc[tmp.majornum == 1, ]
            # set the update
            tmpdf = tmp
        
        # the award levels to keep
        if level_keep is not None:
            assert isinstance(level_keep, list), 'level_keep must be a list'
            if len(level_keep) > 0:
                tmp = tmpdf
                tmp = tmp.loc[tmp.awlevel.isin(level_keep), ]
                # set the update
                tmpdf = tmp
            
        # filter the columns
        if cols is not None:
            assert isinstance(cols, list), 'cols must be a list'
            if len(cols) > 0:
                tmp = tmpdf
                tmp_f = tmp >> select(cols)
                tmpdf = tmp_f
        
        # return the dataset
        self.df = tmpdf


## another class






import pickle
import pandas as pd

def comp_graph1():
    """
    Returns a dictionary of dataframes that flags "similar" schools and majors offered for graph-based analysis.
    """

    edges_url = "https://raw.githubusercontent.com/Btibert3/datasets/master/he-competitor-graphs/site1-links.csv"
    majors_url = "https://raw.githubusercontent.com/Btibert3/datasets/master/he-competitor-graphs/site1-majors.csv"
    edges = pd.read_csv(edges_url)
    majors = pd.read_csv(majors_url)
    comp1 = dict(edges=edges, majors=majors)
    return(comp1)


def comp_graph2():
    """
    Returns a dictionary of dataframes for graph analysis of school competition and mission statements.
    """

    edges_url = "https://raw.githubusercontent.com/Btibert3/datasets/master/he-competitor-graphs/site2-edges.csv"
    mission_url = "https://raw.githubusercontent.com/Btibert3/datasets/master/he-competitor-graphs/site2-mission.csv"
    edges = pd.read_csv(edges_url)
    mission = pd.read_csv(mission_url)
    comp2 = dict(edges=edges, mission=mission)
    return(comp2)


def comp_graph3():
    """
    Returns a dataframe which represents the "Similar Colleges" for each profile page from a popular College Search site.
    The crawl represents the edges betweeen two institutions, the rank of the edge, and the year the data was 
    collected.
    The data were collected in April 2016 and August of 2019, and can be used as a directed graph to study the
    "structure" of how colleges in the U.S. compete, and more broadly, for various machine learning tasks.
    """

    edges_url = "https://raw.githubusercontent.com/Btibert3/datasets/master/he-competitor-graphs/competition.csv"
    edges = pd.read_csv(edges_url)
    return(edges)




def wiche():
    """
    Returns a dataframe with the most recent WICHE projections in long format.
    """

    url = "https://raw.githubusercontent.com/Btibert3/datasets/master/wiche-hs-grads/wiche-hs-grads.csv"
    wiche_df = pd.read_csv(url)
    return(wiche_df)


def scorecard():
    """
    Returns a dataframe of the most recent college scorecard dataset.

    The Scorecard dataset, not the full dataset.  For the full, use the scorecard_full method.
    """

    url = "https://ed-public-download.app.cloud.gov/downloads/Most-Recent-Cohorts-Scorecard-Elements.csv"
    x = pd.read_csv(url)
    return(x)


def scorecard_full():
    """
    Returns a dataframe of the most recent FULL college scorecard dataset.

    This will take ~ 10 seconds using free online resources, but also asks for a full data download at present.
    Curently, caching is not used but should be.
    """

    url = "https://ed-public-download.app.cloud.gov/downloads/Most-Recent-Cohorts-All-Data-Elements.csv"
    x = pd.read_csv(url)
    return(x)


def scorecard_nslds():
    """
    Returns a dataframe of the most recent cohort for the NSLDS dataset.

    This will take ~ 10 seconds using free online resources, but also asks for a full data download at present.
    Curently, caching is not used but should be.
    """

    url = "https://ed-public-download.app.cloud.gov/downloads/Most-Recent-Cohorts-NSLDS-Elements.csv"
    x = pd.read_csv(url)
    return(x)


def scorecard_earnings():
    """
    Returns a dataframe of the most recent cohort for post school earnings.
    """

    url = "https://ed-public-download.app.cloud.gov/downloads/Most-Recent-Cohorts-Treasury-Elements.csv"
    x = pd.read_csv(url)
    return(x)


def crosswalk():
    """
    Returns a dataframe that attempts to provide a crosswalk between unitids from IPEDS to other datasources traditionally used in higher ed datasets.

    This will take ~ 10 seconds using free online resources, but also asks for a full data download at present.
    Curently, caching is not used but should be.
    """

    url ="https://docs.google.com/spreadsheets/d/e/2PACX-1vQifjGzZDfaTW01tmr3mp_qQ7Om279Dr1sFuSLXEXhcaOjpN5kXwQs6Mpvl9D11nGiIMrQ-asmsLlsk/pub?gid=713012050&single=true&output=csv"
    x = pd.read_csv(url)
    return(x)

def closings():
    """
    Returns a dataframe that is based on a dataset/article from Inside Higher Ed on June 13, 2019

    Source: https://www.insidehighered.com/news/2019/06/13/list-private-colleges-have-closed-recent-years
    """

    url = "https://docs.google.com/spreadsheets/d/e/2PACX-1vTGVWo_yHFEMMcDEk-92EbKNCUKLw32wmyZ0-nkmpadL4Yky_SgHBJOhyKM7uOhHD9nMLRB0s1XqCR6/pub?gid=0&single=true&output=csv"
    x = pd.read_csv(url)
    return(x)

def region_xwalk():
    """
    Returns a dataframe that can be used to map states and regions
    """

    url = "https://docs.google.com/spreadsheets/d/e/2PACX-1vQ62ZENWQnUf2XnRs7hiVn7XhXhuCdZdeEOK2-BgkhppEI_A0IMepafWx9vaenOdhQptz5HIwxq3ZAM/pub?gid=0&single=true&output=csv"
    x = pd.read_csv(url)
    x.columns = x.columns.str.lower()
    return(x)
    
def cipcodes():
    """
    Returns a dataframe of 2010 CIP Codes

    Source: The data dictionary for the Completions A survey
    """

    url = "https://docs.google.com/spreadsheets/d/e/2PACX-1vSjBizdy0EdtUllMJVe0vtq4TIXzsLSR0hnpEyS31-ASx5zjEBkfgLLqOjaHRCtYxqWEVs8eqY0KWJF/pub?gid=0&single=true&output=csv"
    x = pd.read_csv(url)
    x.columns = x.columns.str.lower()
    return(x)

def award_levels():
    """
    Returns a dataframe of award levels for the Completions A Survey

    Source: The 2018 survey data dictionary for the Completions A survey
    """

    url = "https://docs.google.com/spreadsheets/d/e/2PACX-1vQVh15c5xErMKdhMea1AIa3jnArpXXlsSY1NSR_laFaYvhlni3C9jP9DKcHkZqNIsAE18zfMPf0qZFu/pub?gid=0&single=true&output=csv"
    x = pd.read_csv(url)
    x.columns = x.columns.str.lower()
    return(x)






# test the c_a class
from pypeds import ipeds


############### Test range of years

# the years to teset
years = list(range(2002, 2019))

# instantiate and go
tmp = ipeds.C_A(years=years)
tmp.extract()
x = tmp.load()
x.shape
x.fall_year.value_counts(dropna=False, sort=False)

## cleanup
del years
del tmp
del x



# test the SFA class
from pypeds import views



############### Test range of years

# simple test
x = views.tuition_discounting()



# larger test
fall_years = list(range(2002, 2018))

# go for it
x = views.tuition_discounting(fall_years=fall_years)


# throws a warning but returns a data
# TODO: what are the columns
# DtypeWarning: Columns (12) have mixed types. 
# Specify dtype option on import or set low_memory=False.
# DtypeWarning: Columns (177,186,195) have mixed types. 
# Specify dtype option on import or set low_memory=False.



#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['pypeds']

package_data = \
{'': ['*'], 'pypeds': ['.ipynb_checkpoints/*']}

install_requires = \
['requests', 'pandas', 'altair', 'dfply', 'numpy']

setup(name='pypeds',
      version='0.102',
      description='A python package to help expedite insight generate from publicly available higher education datasets. ',
      author='Brock Tibert',
      author_email='btibert3@gmail.com',
      url='https://github.com/Btibert3/pypeds',
      packages=packages,
      package_data=package_data,
      install_requires=install_requires,
      python_requires='>=3.6',
     )



# layer above ETL framework - views are tasks to build specific datasets
import pandas as pd
from dfply import *
from pypeds import ipeds
from pypeds import datasets


#================================================== migration dataset
## the migration data, with school and residence region data appended
def migration(years=[2018], 
              efc_line = list(range(1,99)),
              efc_cols = ['unitid', 'fall_year', 'line', 'efres02'],
              hd_deg4yr = True,
              hd_service = True,
              hd_lower48 = None,
              hd_cols = ['unitid', 'fall_year', 'instnm', 
                         'fips', 'obereg', 'sector', 'latitude', 
                         'longitud'] ):
  """
  Build a migration dataset, with common data mappings using the lower
  level API.

  Parameters:
      years (list): a list of integers for the survey years to include for the migration data
      efc_line (list): a list of integers representing the values for the line field in efc
      efc_cols (list): a list of valid column names from the efc survey to keep
      hd_deg4yr (bool): boolean (default = True) as to filter to only include degree-granting 4-year institutions
      hd_service (bool): boolean (default = True) which if True, will remove US service schools
      hd_lower48 (bool): boolean (default = None) while if True, will only keep lower 48 states
      hd_cols (list): a list of valid column names for the HD survey.  Only these columns will be returned.
  """
  
  # get the migration data for the years parameter
  m = ipeds.EFC(years=years)
  m.extract()
  m.transform(line=efc_line)
  m.transform(cols=efc_cols)
  m = m.load()
  
  # get the inst data
  i = ipeds.HD(years=years)
  i.extract()
  i.transform(deg4yr=hd_deg4yr)
  i.transform(service=hd_service)
  i.transform(lower_us=hd_lower48)
  i.transform(cols=hd_cols)
  inst = i.load()
  
  # the region dataset
  r = datasets.region_xwalk()
  
  # join the inst data onto migration
  # inner join to keep the school filters
  df = pd.merge(left=m, right=inst, on=['unitid','fall_year'], how='inner')
  
  # merge on data about the school region
  r1 = r >> select(['fips', 'name', 'ipeds_region'])
  df = pd.merge(left=df, right=r1, left_on='fips', right_on='fips', how='left')
  
  # merge on the region info about the state of residence
  r2 = r >> select(['ipeds_code', 'name', 'ipeds_region', 'region', 'division'])
  df = pd.merge(left=df, right=r2, left_on='line', right_on='ipeds_code', how='left', suffixes=['_inst', '_state'])
  
  # return the data
  return (df)



#================================================== discounting dataset
## private institution tuition discounting
def tuition_discounting(fall_years = [2017],
                        hd_deg4yr = True,
                        hd_service = True,
                        hd_lower48 = None,
                        hd_cols = ['unitid', 'fall_year', 'instnm', 'fips', 
                                   'carnegie', 'sector', 'latitude', 'longitud'],
                        sfa_cols =['unitid', 
                                   'fall_year',  
                                   'scfa1n', 
                                   'anyaidn','anyaidp', 
                                   'igrnt_n', 'igrnt_p', 'igrnt_a', 
                                   'fgrnt_a', 'sgrnt_a', 'loan_a'],
                        icay_cols = ['unitid',
                                     'fall_year',
                                     'chg2ay3',
                                     'chg4ay3',
                                     'chg5ay3',
                                     'chg6ay3'],
                        ff2_cols = ['unitid',
                                    'fall_year',
                                    'f2d01',
                                    'f2c08',
                                    'f2h01',
                                    'f2h02']):
  """
  Build a tuition tuition discounting dataset

  Parameters:
      fall_years (list): a list of integers for the Fall years to include for the migration data.  For example, 2017 is Fall 2017.
      hd_deg4yr (bool): boolean (default = True) as to filter to only include degree-granting 4-year institutions
      hd_service (bool): boolean (default = True) which if True, will remove US service schools
      hd_lower48 (bool): boolean (default = None) while if True, will only keep lower 48 states
      hd_cols (list): a list of valid column names for the HD survey.  Only these columns will be returned.
      sfa_cols (list): a list of valid column names for the SFA survey.  Only these columns will be returned.
      icay_cols (list): a list of valid column names for the ICAY survey.  Only these columns will be returned.
      ff2_cols (list): a list of valid column names for FASB survey. Only these columns will be returned
  """ 
  
  # the schools
  i = ipeds.HD(years=fall_years)
  i.extract()
  i.transform(deg4yr=hd_deg4yr)
  i.transform(service=hd_service)
  i.transform(lower_us=hd_lower48)
  i.transform(cols=hd_cols)
  inst = i.load()
  
  # keep only privates
  inst = inst.loc[inst.sector == 2, ]
  
  # the student finaid data
  # add one because the aid for the fall data is released a year later
  years = list(np.array(fall_years) + 1)
  s = ipeds.SFA(years=years)
  s.extract()
  s.transform(cols=sfa_cols)
  aid = s.load()
  
  # the charges 
  c = ipeds.ICAY(years = fall_years)
  c.extract()
  c.transform(cols=icay_cols)
  charges = c.load()
  
  # the private FASB data
  f =  ipeds.FF2(years = years)
  f.extract()
  f.transform(cols=ff2_cols)
  fin = f.load()
  
  # merge the datasets
  df = pd.merge(inst, aid, on=['unitid','fall_year'], how="left")
  df = pd.merge(df, charges, on=['unitid','fall_year'], how="left")
  df = pd.merge(df, fin, on=['unitid','fall_year'], how="left")
  
  ##### need to change charges 
  df.chg2ay3 = pd.to_numeric(df['chg2ay3'], errors='coerce')
  df.chg4ay3 = pd.to_numeric(df['chg4ay3'], errors='coerce')
  df.chg5ay3 = pd.to_numeric(df['chg5ay3'], errors='coerce')
  df.chg6ay3 = pd.to_numeric(df['chg6ay3'], errors='coerce')
  
  # add the metrics
  df['discount'] = df.f2c08 / (df.f2c08 + df.f2d01)
  df['anyaid_pct'] = df.anyaidp / 100
  df['instaid_pct'] = df.igrnt_p / 100
  df['net_tuition'] = df.chg2ay3 - df.igrnt_a
  df['aided_student_disc'] = 1 - (df.net_tuition / df.chg2ay3)
  df['aid_pct_tuitfee'] = df.igrnt_a / df.chg2ay3
  df['tuition_disc'] = df.instaid_pct * df.aid_pct_tuitfee
  # total aid for a student geting the average inst, federal, state, loan
  df['total_aid'] = df.igrnt_a + df.fgrnt_a + df.sgrnt_a + df.loan_a
  df['total_charges'] = df.chg2ay3 + df.chg4ay3 + df.chg5ay3 + df.chg6ay3
  
  # return the dataset
  return(df)


#================================================== completions by program
## a dataframe with school and completions by program data
def program_completions(fall_years=[2017],
                        hd_deg4yr = True,
                        hd_service = False,
                        hd_lower48 = False,
                        hd_regions = True,
                        hd_cols = ['unitid', 'fall_year', 'instnm', 'fips', 
                                   'carnegie', 'sector', 'latitude', 'longitud'],
                        degree_code = [5,7]):
  """
  Build a dataset of school info and program completions.  

  Parameters:
      years (list): a list of integers for the survey years to include for the migration data
      hd_deg4yr (bool): boolean (default = True) as to filter to only include degree-granting 4-year institutions
      hd_service (bool): boolean (default = True) which if True, will remove US service schools
      hd_lower48 (bool): boolean (default = False) which if True, will only keep lower 48 states
      hd_regions (bool): boolean (default = True) which if True, will add state and region data at the institutional level
      hd_cols (list): a list of valid column names for the HD survey.  Only these columns will be returned.
      degree_code (list): a list of valid values for the awlevel column in C_A (completions by program)
  """

  
  # get the inst data
  i = ipeds.HD(years=fall_years)
  i.extract()
  i.transform(deg4yr=hd_deg4yr)
  i.transform(service=hd_service)
  i.transform(lower_us=hd_lower48)
  i.transform(regions=hd_regions)
  i.transform(cols=hd_cols)
  inst = i.load()

  # the completions for the academic year are reported a year later
  years = list(np.array(fall_years) + 1)
  c = ipeds.C_A(years=years)
  c.extract()
  c.transform(level_keep=degree_code)
  comps = c.load()
  
  # merge the data together
  # only those that match
  df = pd.merge(inst, comps, on=['unitid', 'fall_year'], how='inner')
  
  # return the data
  return (df)




#================================================== another view
## the description
  
  
  
  
  
  
  
